@@ -97,6 +97,8 @@ ldbm_back_add(Slapi_PBlock *pb)
     PRUint64 conn_id;
     int op_id;
     int result_sent = 0;
+    int32_t parent_op = 0;
+    struct timespec parent_time;
 
     if (slapi_pblock_get(pb, SLAPI_CONN_ID, &conn_id) < 0) {
         conn_id = 0; /* connection is NULL */
@@ -147,6 +149,13 @@ ldbm_back_add(Slapi_PBlock *pb)
     slapi_entry_delete_values(e, numsubordinates, NULL);
 
     dblayer_txn_init(li, &txn);
+
+    if (txn.back_txn_txn == NULL) {
+        /* This is the parent operation, get the time */
+        parent_op = 1;
+        parent_time = slapi_current_rel_time_hr();
+    }
+
     /* the calls to perform searches require the parent txn if any
        so set txn to the parent_txn until we begin the child transaction */
     if (parent_txn) {
@@ -857,7 +866,7 @@ ldbm_back_add(Slapi_PBlock *pb)
                 }
                 retval = parent_update_on_childchange(&parent_modify_c, op, NULL);
                 slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_add",
-                              "conn=%lu op=%d parent_update_on_childchange: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
+                              "conn=%" PRIu64 " op=%d parent_update_on_childchange: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
                               conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, retval);
                 /* The modify context now contains info needed later */
                 if (retval) {
@@ -905,10 +914,10 @@ ldbm_back_add(Slapi_PBlock *pb)
                 goto error_return;
             }
         }
-        if (is_tombstone_operation) {
+        if (is_tombstone_operation && slapi_is_loglevel_set(SLAPI_LOG_DEBUG)) {
             int len = 0;
             const char *rs = slapi_entry_get_rdn_const(addingentry->ep_entry);
-            char *es= slapi_entry2str_with_options(e, &len, SLAPI_DUMP_STATEINFO | SLAPI_DUMP_UNIQUEID);
+            char *es= slapi_entry2str_with_options(addingentry->ep_entry, &len, SLAPI_DUMP_STATEINFO | SLAPI_DUMP_UNIQUEID);
             slapi_log_err(SLAPI_LOG_DEBUG, "ldbm_back_add", "now adding entry: %s\n %s\n", rs?rs:"no rdn", es);
             slapi_ch_free_string(&es);
         }
@@ -1053,7 +1062,7 @@ ldbm_back_add(Slapi_PBlock *pb)
             /* Push out the db modifications from the parent entry */
             retval = modify_update_all(be, pb, &parent_modify_c, &txn);
             slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_add",
-                          "conn=%lu op=%d modify_update_all: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
+                          "conn=%" PRIu64 " op=%d modify_update_all: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
                           conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, retval);
             if (DB_LOCK_DEADLOCK == retval) {
                 slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 6 DEADLOCK\n");
@@ -1177,7 +1186,7 @@ ldbm_back_add(Slapi_PBlock *pb)
         /* switch the parent entry copy into play */
         myrc = modify_switch_entries(&parent_modify_c, be);
         slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_add",
-                      "conn=%lu op=%d modify_switch_entries: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
+                      "conn=%" PRIu64 " op=%d modify_switch_entries: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
                       conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, myrc);
         if (0 == myrc) {
             parent_switched = 1;
@@ -1239,6 +1248,10 @@ ldbm_back_add(Slapi_PBlock *pb)
     goto common_return;
 
 error_return:
+    /* Revert the caches if this is the parent operation */
+    if (parent_op) {
+        revert_cache(inst, &parent_time);
+    }
     if (addingentry_id_assigned) {
         next_id_return(be, addingentry->ep_id);
     }
@@ -1257,7 +1270,7 @@ error_return:
          */
         myrc = modify_unswitch_entries(&parent_modify_c, be);
         slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_add",
-                      "conn=%lu op=%d modify_unswitch_entries: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
+                      "conn=%" PRIu64 " op=%d modify_unswitch_entries: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
                       conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, myrc);
     }
 diskfull_return:
@@ -1394,7 +1407,7 @@ common_return:
         modify_term(&ruv_c, be);
     }
     slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_add",
-                  "conn=%lu op=%d modify_term: old_entry=0x%p, new_entry=0x%p\n",
+                  "conn=%" PRIu64 " op=%d modify_term: old_entry=0x%p, new_entry=0x%p\n",
                   conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry);
     myrc = modify_term(&parent_modify_c, be);
     done_with_pblock_entry(pb, SLAPI_ADD_EXISTING_DN_ENTRY);
