@@ -1,5 +1,6 @@
 # --- BEGIN COPYRIGHT BLOCK ---
 # Copyright (C) 2016 Red Hat, Inc.
+# Copyright (C) 2019 William Brown <william@blackhats.net.au>
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -79,7 +80,7 @@ def _warn(data, msg=None):
     return data
 
 
-def connect_instance(dsrc_inst, verbose):
+def connect_instance(dsrc_inst, verbose, args):
     dsargs = dsrc_inst['args']
     if '//' not in dsargs['ldapurl']:
         # Connecting to the local instance
@@ -97,12 +98,38 @@ def connect_instance(dsrc_inst, verbose):
         else:
             # The instance name does not match any instances
             raise ValueError("Could not find configuration for instance: " + dsargs['ldapurl'])
+
     ds = DirSrv(verbose=verbose)
+    # We do an empty allocate here to determine if we can autobind ... (really
+    # we should actually be inspect the URL ...)
     ds.allocate(dsargs)
-    if not ds.can_autobind() and dsrc_inst['binddn'] is not None:
+
+    if args.pwdfile is not None or args.bindpw is not None or args.prompt is True:
+        if args.pwdfile is not None:
+            # Read password from file
+            try:
+                with open(args.pwdfile, "r") as f:
+                    dsargs[SER_ROOT_PW] = f.readline().rstrip()
+            except EnvironmentError as e:
+                raise ValueError("Failed to open password file: " + str(e))
+        elif args.bindpw is not None:
+            # Password provided
+            # This shouldn't be needed? dsrc already inherits the args ...
+            dsargs[SER_ROOT_PW] = args.bindpw
+        else:
+            # No password or we chose to prompt
+            dsargs[SER_ROOT_PW] = getpass("Enter password for {} on {}: ".format(dsrc_inst['binddn'], dsrc_inst['uri']))
+    elif not ds.can_autobind():
+        # No LDAPI, prompt for password
         dsargs[SER_ROOT_PW] = getpass("Enter password for {} on {}: ".format(dsrc_inst['binddn'], dsrc_inst['uri']))
-    elif not ds.can_autobind() and dsrc_inst['binddn'] is None:
-        raise Exception("Must provide a binddn to connect with")
+
+    if 'binddn' in dsrc_inst:
+        # Allocate is an awful interface that we should stop using, but for now
+        # just directly map the dsrc_inst args in (remember, dsrc_inst DOES
+        # overlay cli args into the map ...)
+        dsargs[SER_ROOT_DN] = dsrc_inst['binddn']
+
+    ds = DirSrv(verbose=verbose)
     ds.allocate(dsargs)
     ds.open(saslmethod=dsrc_inst['saslmech'],
             certdir=dsrc_inst['tls_cacertdir'],
@@ -140,7 +167,7 @@ def _generic_list(inst, basedn, log, manager_class, args=None):
             if args and args.json:
                 json_result['items'].append(o_str)
             else:
-                log.info(o_str)
+                print(o_str)
         if args and args.json:
             print(json.dumps(json_result))
 
@@ -154,7 +181,7 @@ def _generic_get(inst, basedn, log, manager_class, selector, args=None):
     else:
         o = mc.get(selector)
         o_str = o.display()
-        log.info(o_str)
+        print(o_str)
 
 
 def _generic_get_entry(inst, basedn, log, manager_class, args=None):
@@ -162,7 +189,7 @@ def _generic_get_entry(inst, basedn, log, manager_class, args=None):
     if args and args.json:
         print(mc.get_all_attrs_json())
     else:
-        log.info(mc.display())
+        print(mc.display())
 
 
 def _generic_get_attr(inst, basedn, log, manager_class, args=None):
@@ -171,28 +198,28 @@ def _generic_get_attr(inst, basedn, log, manager_class, args=None):
         if args and args.json:
             print(mc.get_attr_vals_json(attr))
         else:
-            log.info(mc.display_attr(attr).rstrip())
+            print(mc.display_attr(attr).rstrip())
 
 
 def _generic_get_dn(inst, basedn, log, manager_class, dn, args=None):
     mc = manager_class(inst, basedn)
     o = mc.get(dn=dn)
     o_str = o.display()
-    log.info(o_str)
+    print(o_str)
 
 
 def _generic_create(inst, basedn, log, manager_class, kwargs, args=None):
     mc = manager_class(inst, basedn)
     o = mc.create(properties=kwargs)
     o_str = o.__unicode__()
-    log.info('Successfully created %s' % o_str)
+    print('Successfully created %s' % o_str)
 
 
 def _generic_delete(inst, basedn, log, object_class, dn, args=None):
     # Load the oc direct
     o = object_class(inst, dn)
     o.delete()
-    log.info('Successfully deleted %s' % dn)
+    print('Successfully deleted %s' % dn)
 
 
 # Attr functions expect attribute values to be "attr=value"
@@ -204,7 +231,7 @@ def _generic_replace_attr(inst, basedn, log, manager_class, args=None):
             if "=" in myattr:
                 [attr, val] = myattr.split("=", 1)
                 mc.replace(attr, val)
-                log.info("Successfully replaced \"{}\"".format(attr))
+                print("Successfully replaced \"{}\"".format(attr))
             else:
                 raise ValueError("You must specify a value to replace the attribute ({})".format(myattr))
     else:
@@ -219,7 +246,7 @@ def _generic_add_attr(inst, basedn, log, manager_class, args=None):
             if "=" in myattr:
                 [attr, val] = myattr.split("=", 1)
                 mc.add(attr, val)
-                log.info("Successfully added \"{}\"".format(attr))
+                print("Successfully added \"{}\"".format(attr))
             else:
                 raise ValueError("You must specify a value to add for the attribute ({})".format(myattr))
     else:
@@ -239,11 +266,52 @@ def _generic_del_attr(inst, basedn, log, manager_class, args=None):
                 # remove all
                 mc.remove_all(myattr)
                 attr = myattr  # for logging
-            log.info("Successfully removed \"{}\"".format(attr))
+            print("Successfully removed \"{}\"".format(attr))
     else:
         # Missing value
         raise ValueError("Missing attribute to delete")
 
+def _generic_modify_change_to_mod(change):
+    values = change.split(":")
+    if len(values) <= 2:
+        raise ValueError("Not enough arguments in '%s'. action:attribute:value" % change)
+    elif len(values) >= 4:
+        raise ValueError("Too many arguments in '%s'. %s:attribute:value expected" % (change, values[0]))
+    elif len(values[1]) == 0:
+        raise ValueError("Invalid empty attribute name in '%s'." % change)
+    # Return a tuple
+    if values[0] == 'add':
+        if len(values[2]) == 0:
+            raise ValueError("Invalid empty value in '%s'." % change)
+        return (ldap.MOD_ADD, values[1], values[2])
+    elif values[0] == 'delete':
+        if len(values[2]) == 0:
+            return (ldap.MOD_DELETE, values[1])
+        return (ldap.MOD_DELETE, values[1], values[2])
+    elif values[0] == 'replace':
+        if len(values[2]) == 0:
+            raise ValueError("Invalid empty value in '%s'." % change)
+        return (ldap.MOD_REPLACE, values[1], values[2])
+    else:
+        raise ValueError("Unknown action '%s'. Expected add, delete, replace" % change)
+
+def _generic_modify(inst, basedn, log, manager_class, selector, args=None):
+    # Here, we should have already selected the type etc. mc should be a
+    # type of DSLdapObject (singular)
+    mc = manager_class(inst, basedn)
+    # Get the object
+    if args and args.changes:
+        o = mc.get(selector)
+        # Now parse the series of arguments.
+        # Turn them into mod lists. See apply_mods.
+        mods = [_generic_modify_change_to_mod(x) for x in args.changes]
+        log.debug("Requested mods: %s" % mods)
+        # Now push them to dsldapobject to modify
+        o.apply_mods(mods)
+        print('Successfully modified %s' % o.dn)
+    else:
+        raise ValueError("Missing modify actions to perform.")
+
 
 class LogCapture(logging.Handler):
     """
