@@ -6,6 +6,7 @@
 # See LICENSE for details.
 # --- END COPYRIGHT BLOCK ---
 
+import collections
 import ldap
 import copy
 import os.path
@@ -13,6 +14,7 @@ import os.path
 from lib389 import tasks
 from lib389._mapped_object import DSLdapObjects, DSLdapObject
 from lib389.lint import DSRILE0001
+from lib389.utils import ensure_str, ensure_list_bytes
 from lib389._constants import DN_PLUGIN
 from lib389.properties import (
         PLUGINS_OBJECTCLASS_VALUE, PLUGIN_PROPNAME_TO_ATTRNAME,
@@ -120,8 +122,22 @@ class AttributeUniquenessPlugin(Plugin):
     :type dn: str
     """
 
+    _plugin_properties = {
+        'nsslapd-pluginEnabled': 'off',
+        'nsslapd-pluginPath': 'libattr-unique-plugin',
+        'nsslapd-pluginInitfunc': 'NSUniqueAttr_Init',
+        'nsslapd-pluginType': 'betxnpreoperation',
+        'nsslapd-plugin-depends-on-type': 'database',
+        'nsslapd-pluginId': 'NSUniqueAttr',
+        'nsslapd-pluginVendor': '389 Project',
+        'nsslapd-pluginVersion': 'none',
+        'nsslapd-pluginDescription': 'Enforce unique attribute values',
+    }
+
     def __init__(self, instance, dn="cn=attribute uniqueness,cn=plugins,cn=config"):
         super(AttributeUniquenessPlugin, self).__init__(instance, dn)
+        self._protected = False
+        self._create_objectclasses = ['top', 'nsslapdplugin', 'extensibleObject']
 
     ## These are some wrappers to the important attributes
     # This plugin will be "tricky" in that it can have "many" instance
@@ -158,6 +174,76 @@ class AttributeUniquenessPlugin(Plugin):
         self.set('uniqueness-across-all-subtrees', 'off')
 
 
+class AttributeUniquenessPlugins(DSLdapObjects):
+    """A DSLdapObjects entity which represents Attribute Uniqueness plugin instances
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn="cn=plugins,cn=config"):
+        super(DSLdapObjects, self).__init__(instance)
+        self._instance = instance
+        self._objectclasses = ['top', 'nsslapdplugin', 'extensibleObject']
+        self._filterattrs = ['cn', 'nsslapd-pluginPath']
+        self._childobject = AttributeUniquenessPlugin
+        self._basedn = basedn
+        # This is used to allow entry to instance to work
+        self._list_attrlist = ['dn', 'nsslapd-pluginPath']
+        self._search_filter = "(nsslapd-pluginId=NSUniqueAttr)"
+        self._scope = ldap.SCOPE_SUBTREE
+        self._server_controls = None
+        self._client_controls = None
+
+    def list(self):
+        """Get a list of all plugin instances where nsslapd-pluginId: NSUniqueAttr
+
+        :returns: A list of children entries
+        """
+
+        try:
+            results = self._instance.search_ext_s(
+                base=self._basedn,
+                scope=ldap.SCOPE_ONELEVEL,
+                filterstr=self._search_filter,
+                attrlist=self._list_attrlist,
+                serverctrls=self._server_controls, clientctrls=self._client_controls
+            )
+            insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
+        except ldap.NO_SUCH_OBJECT:
+            # There are no objects to select from, se we return an empty array
+            insts = []
+        return insts
+
+    def _get_dn(self, dn):
+        # This will yield and & filter for objectClass with as many terms as needed.
+        self._log.debug('_gen_dn filter = %s' % self._search_filter)
+        self._log.debug('_gen_dn dn = %s' % dn)
+        return self._instance.search_ext_s(
+            base=dn,
+            scope=ldap.SCOPE_BASE,
+            filterstr=self._search_filter,
+            attrlist=self._list_attrlist,
+            serverctrls=self._server_controls, clientctrls=self._client_controls
+        )
+
+    def _get_selector(self, selector):
+        # Filter based on the objectclasses and the basedn
+        # Based on the selector, we should filter on that too.
+        # This will yield and & filter for objectClass with as many terms as needed.
+        filterstr = "(&(cn=%s)%s)" % (selector, self._search_filter)
+        self._log.debug('_gen_selector filter = %s' % filterstr)
+        return self._instance.search_ext_s(
+            base=self._basedn,
+            scope=self._scope,
+            filterstr=filterstr,
+            attrlist=self._list_attrlist,
+            serverctrls=self._server_controls, clientctrls=self._client_controls
+        )
+
+
 class LdapSSOTokenPlugin(Plugin):
     """An instance of ldapssotoken plugin entry
 
@@ -192,7 +278,7 @@ class ManagedEntriesPlugin(Plugin):
     :type dn: str
     """
 
-    def __init__(self, instance, dn="cn=managed entries,cn=plugins,cn=config"):
+    def __init__(self, instance, dn="cn=Managed Entries,cn=plugins,cn=config"):
         super(ManagedEntriesPlugin, self).__init__(instance, dn)
 
 
@@ -222,11 +308,14 @@ class MEPConfigs(DSLdapObjects):
     :type basedn: str
     """
 
-    def __init__(self, instance, basedn="cn=managed entries,cn=plugins,cn=config"):
+    def __init__(self, instance, basedn=None):
         super(MEPConfigs, self).__init__(instance)
         self._objectclasses = ['top', 'extensibleObject']
         self._filterattrs = ['cn']
         self._childobject = MEPConfig
+        # So we can set the configArea easily
+        if basedn is None:
+            basedn = "cn=managed entries,cn=plugins,cn=config"
         self._basedn = basedn
 
 
@@ -243,7 +332,7 @@ class MEPTemplate(DSLdapObject):
         super(MEPTemplate, self).__init__(instance, dn)
         self._rdn_attribute = 'cn'
         self._must_attributes = ['cn']
-        self._create_objectclasses = ['top', 'extensibleObject', 'mepTemplateEntry']
+        self._create_objectclasses = ['top', 'mepTemplateEntry']
         self._protected = False
 
 
@@ -258,7 +347,7 @@ class MEPTemplates(DSLdapObjects):
 
     def __init__(self, instance, basedn):
         super(MEPTemplates, self).__init__(instance)
-        self._objectclasses = ['top', 'extensibleObject']
+        self._objectclasses = ['top', 'mepTemplateEntry']
         self._filterattrs = ['cn']
         self._childobject = MEPTemplate
         self._basedn = basedn
@@ -774,6 +863,23 @@ class MemberOfSharedConfig(DSLdapObject):
         self._exit_code = None
 
 
+class MemberOfSharedConfigs(DSLdapObjects):
+    """A DSLdapObjects entity which represents MemberOf config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn=None):
+        super(MemberOfSharedConfigs, self).__init__(instance)
+        self._objectclasses = ['top', 'extensibleObject']
+        self._filterattrs = ['cn']
+        self._childobject = MemberOfSharedConfig
+        self._basedn = basedn
+
+
 class RetroChangelogPlugin(Plugin):
     """An instance of Retro Changelog plugin entry
 
@@ -884,6 +990,24 @@ class AutoMembershipPlugin(Plugin):
         return task
 
 
+class AutoMembershipRegexRule(DSLdapObject):
+    def __init__(self, instance, dn=None):
+        super(AutoMembershipRegexRule, self).__init__(instance, dn)
+        self._rdn_attribute = 'cn'
+        self._must_attributes = ['cn', 'autoMemberTargetGroup']
+        self._create_objectclasses = ['top', 'autoMemberRegexRule']
+        self._protected = False
+
+
+class AutoMembershipRegexRules(DSLdapObjects):
+    def __init__(self, instance, basedn="cn=Auto Membership Plugin,cn=plugins,cn=config"):
+        super(AutoMembershipRegexRules, self).__init__(instance)
+        self._objectclasses = ['top', 'autoMemberRegexRule']
+        self._filterattrs = ['cn']
+        self._childobject = AutoMembershipRegexRule
+        self._basedn = basedn
+
+
 class AutoMembershipDefinition(DSLdapObject):
     """A single instance of Auto Membership Plugin config entry
 
@@ -897,7 +1021,7 @@ class AutoMembershipDefinition(DSLdapObject):
         super(AutoMembershipDefinition, self).__init__(instance, dn)
         self._rdn_attribute = 'cn'
         self._must_attributes = ['cn', 'autoMemberScope', 'autoMemberFilter', 'autoMemberGroupingAttr']
-        self._create_objectclasses = ['top', 'AutoMemberDefinition']
+        self._create_objectclasses = ['top', 'autoMemberDefinition']
         self._protected = False
 
     def get_groupattr(self):
@@ -918,7 +1042,7 @@ class AutoMembershipDefinition(DSLdapObject):
     def set_defaultgroup(self, attr):
         """Set autoMemberDefaultGroup attribute"""
 
-        self.set('autoMemberDefaultGroup', attr)  
+        self.set('autoMemberDefaultGroup', attr)
 
     def get_scope(self, attr):
         """Get autoMemberScope attributes"""
@@ -940,6 +1064,42 @@ class AutoMembershipDefinition(DSLdapObject):
 
         self.set('autoMemberFilter', attr)
 
+    def add_regex_rule(self, rule_name, target, include_regex=None, exclude_regex=None):
+        """Add a regex rule
+        :param rule_name - Name of the rule - used dfor the "cn" value inthe DN of the rule entry
+        :param target - the target group DN
+        :param include_regex - a List of regex rules used for group inclusion
+        :param exclude_regex - a List of regex rules used for group exclusion
+        """
+        props = {'cn': rule_name,
+                 'autoMemberTargetGroup': target}
+
+        if include_regex is not None:
+            props['autoMemberInclusiveRegex'] = include_regex
+        if exclude_regex is not None:
+            props['autoMemberInclusiveRegex'] = exclude_regex
+
+        rules = AutoMembershipRegexRules(self._instance, basedn=self.dn)
+        rules.create(properties=props)
+
+    def del_regex_rule(self, rule_name):
+        """Delete a regex rule from this definition
+        :param rule_name - The "cn" values of the regex rule entry
+        :raises ValueError - If a regex rule entry can not be found using rule_name
+        """
+        rules = AutoMembershipRegexRules(self._instance, basedn=self.dn)
+        regex = rules.get(selector=rule_name)
+        if regex is not None:
+            regex.delete()
+        else:
+            raise ValueError("No regex rule found with the name ({}) under ({})".format(rule_name, self.dn))
+
+    def list_regex_rules(self):
+        """Return a list of regex rule entries for this definition
+        """
+        rules = AutoMembershipRegexRules(self._instance, basedn=self.dn)
+        return rules.list()
+
 
 class AutoMembershipDefinitions(DSLdapObjects):
     """A DSLdapObjects entity which represents Auto Membership Plugin config entry
@@ -958,6 +1118,40 @@ class AutoMembershipDefinitions(DSLdapObjects):
         self._basedn = basedn
 
 
+class AutoMembershipRegexRule(DSLdapObject):
+    """A single instance of Auto Membership Plugin Regex Rule config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn=None):
+        super(AutoMembershipRegexRule, self).__init__(instance, dn)
+        self._rdn_attribute = 'cn'
+        self._must_attributes = ['cn', 'autoMemberTargetGroup']
+        self._create_objectclasses = ['top', 'autoMemberRegexRule']
+        self._protected = False
+
+
+class AutoMembershipRegexRules(DSLdapObjects):
+    """A DSLdapObjects entity which represents Auto Membership Plugin Regex Rule config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn):
+        super(AutoMembershipRegexRules, self).__init__(instance)
+        self._objectclasses = ['top', 'autoMemberRegexRule']
+        self._filterattrs = ['cn']
+        self._childobject = AutoMembershipRegexRule
+        self._basedn = basedn
+
+
 class ContentSynchronizationPlugin(Plugin):
     """A single instance of Content Synchronization plugin entry
 
@@ -1009,21 +1203,19 @@ class LinkedAttributesPlugin(Plugin):
     def __init__(self, instance, dn="cn=Linked Attributes,cn=plugins,cn=config"):
         super(LinkedAttributesPlugin, self).__init__(instance, dn)
 
-    def fixup(self, basedn, _filter=None):
+    def fixup(self, linkdn):
         """Create a fixup linked attributes task
 
-        :param basedn: Basedn to fix up
-        :type basedn: str
-        :param _filter: a filter for entries to fix up
-        :type _filter: str
+        :param linkdn: Link DN to fix up
+        :type linkdn: str
 
         :returns: an instance of Task(DSLdapObject)
         """
 
         task = tasks.FixupLinkedAttributesTask(self._instance)
-        task_properties = {'basedn': basedn}
-        if _filter is not None:
-            task_properties['filter'] = _filter
+        task_properties = {}
+        if linkdn is not None:
+            task_properties['linkdn'] = linkdn
         task.create(properties=task_properties)
 
         return task
@@ -1059,6 +1251,7 @@ class LinkedAttributesConfigs(DSLdapObjects):
         super(LinkedAttributesConfigs, self).__init__(instance)
         self._objectclasses = ['top', 'extensibleObject']
         self._filterattrs = ['cn']
+        self._scope = ldap.SCOPE_ONELEVEL
         self._childobject = LinkedAttributesConfig
         self._basedn = basedn
 
@@ -1075,6 +1268,93 @@ class PassThroughAuthenticationPlugin(Plugin):
     def __init__(self, instance, dn="cn=Pass Through Authentication,cn=plugins,cn=config"):
         super(PassThroughAuthenticationPlugin, self).__init__(instance, dn)
 
+    def get_urls(self):
+        """Get all URLs from nsslapd-pluginargNUM attributes
+
+        :returns: a list
+        """
+
+        attr_dict = collections.OrderedDict(sorted(self.get_all_attrs().items()))
+        result = []
+        for attr, value in attr_dict.items():
+            if attr.startswith("nsslapd-pluginarg"):
+                result.append(ensure_str(value[0]))
+        return result
+
+
+class POSIXWinsyncPlugin(Plugin):
+    """A single instance of Posix Winsync API plugin entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn="cn=Posix Winsync API,cn=plugins,cn=config"):
+        super(POSIXWinsyncPlugin, self).__init__(instance, dn)
+
+
+class PAMPassThroughAuthPlugin(Plugin):
+    """A single instance of PAM Pass Through Auth plugin entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn="cn=PAM Pass Through Auth,cn=plugins,cn=config"):
+        super(PAMPassThroughAuthPlugin, self).__init__(instance, dn)
+
+
+class PAMPassThroughAuthConfig(Plugin):
+    """A single instance of PAM Pass Through Auth config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    _plugin_properties = {
+        'cn' : 'USN',
+        'nsslapd-pluginEnabled': 'off',
+        'nsslapd-pluginPath': 'libpam-passthru-plugin',
+        'nsslapd-pluginInitfunc': 'pam_passthruauth_init',
+        'nsslapd-pluginType': 'betxnpreoperation',
+        'nsslapd-plugin-depends-on-type': 'database',
+        'nsslapd-pluginId': 'PAM',
+        'nsslapd-pluginVendor': '389 Project',
+        'nsslapd-pluginVersion': '1.3.7.0',
+        'nsslapd-pluginDescription': 'PAM Pass Through Auth plugin'
+    }
+
+    def __init__(self, instance, dn=None):
+        super(PAMPassThroughAuthConfig, self).__init__(instance, dn)
+        self._rdn_attribute = 'cn'
+        self._must_attributes = ['cn']
+        self._create_objectclasses = ['top', 'extensibleObject', 'nsslapdplugin', 'pamConfig']
+        self._protected = False
+
+
+class PAMPassThroughAuthConfigs(DSLdapObjects):
+    """A DSLdapObjects entity which represents PAM Pass Through Auth config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn="cn=PAM Pass Through Auth,cn=plugins,cn=config"):
+        super(PAMPassThroughAuthConfigs, self).__init__(instance)
+        self._objectclasses = ['top', 'extensibleObject', 'nsslapdplugin', 'pamConfig']
+        self._filterattrs = ['cn']
+        self._scope = ldap.SCOPE_ONELEVEL
+        self._childobject = PAMPassThroughAuthConfig
+        self._basedn = basedn
+
 
 class USNPlugin(Plugin):
     """A single instance of USN (Update Sequence Number) plugin entry
@@ -1541,8 +1821,109 @@ class DNAPluginConfigs(DSLdapObjects):
         self._basedn = basedn
 
 
+class DNAPluginSharedConfig(DSLdapObject):
+    """A single instance of DNA Plugin config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn=None):
+        super(DNAPluginSharedConfig, self).__init__(instance, dn)
+        self._rdn_attribute = 'dnaHostname'
+        self._must_attributes = ['dnaHostname', 'dnaPortNum']
+        self._create_objectclasses = ['top', 'dnaSharedConfig']
+        self._protected = False
+
+    def create(self, properties=None, basedn=None, ensure=False):
+        """The shared config DNA plugin entry has two RDN values
+        The function takes care about that special case
+        """
+
+        for attr in self._must_attributes:
+            if properties.get(attr, None) is None:
+                raise ldap.UNWILLING_TO_PERFORM('Attribute %s must not be None' % attr)
+
+        assert basedn is not None, "Base DN should be specified"
+
+        # Make a DN with the two items RDN and base DN
+        decomposed_dn = [[('dnaHostname', properties['dnaHostname'], 1),
+                          ('dnaPortNum', properties['dnaPortNum'], 1)]] + ldap.dn.str2dn(basedn)
+        dn = ldap.dn.dn2str(decomposed_dn)
+
+        exists = False
+        if ensure:
+            # If we are running in stateful ensure mode, we need to check if the object exists, and
+            # we can see the state that it is in.
+            try:
+                self._instance.search_ext_s(dn, ldap.SCOPE_BASE, self._object_filter, attrsonly=1,
+                                            serverctrls=self._server_controls, clientctrls=self._client_controls,
+                                            escapehatch='i am sure')
+                exists = True
+            except ldap.NO_SUCH_OBJECT:
+                pass
+
+        if exists and ensure:
+            # update properties
+            self._log.debug('Exists %s' % dn)
+            self._dn = dn
+            # Now use replace_many to setup our values
+            mods = []
+            for k, v in list(properties.items()):
+                mods.append((ldap.MOD_REPLACE, k, v))
+            self._instance.modify_ext_s(self._dn, mods, serverctrls=self._server_controls,
+                                        clientctrls=self._client_controls, escapehatch='i am sure')
+        else:
+            self._log.debug('Creating %s' % dn)
+            mods = [('objectclass', ensure_list_bytes(self._create_objectclasses))]
+            # Bring our mods to one type and do ensure bytes on the list
+            for attr, value in properties.items():
+                if not isinstance(value, list):
+                    value = [value]
+                mods.append((attr, ensure_list_bytes(value)))
+            # We rely on exceptions here to indicate failure to the parent.
+            self._log.debug('Creating entry %s : %s' % (dn, mods))
+            self._instance.add_ext_s(dn, mods, serverctrls=self._server_controls, clientctrls=self._client_controls,
+                                     escapehatch='i am sure')
+            # If it worked, we need to fix our instance dn
+            self._dn = dn
+
+        return self
+
+
+class DNAPluginSharedConfigs(DSLdapObjects):
+    """A DSLdapObjects entity which represents DNA Plugin config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn=None):
+        super(DNAPluginSharedConfigs, self).__init__(instance)
+        self._objectclasses = ['top', 'dnaSharedConfig']
+        self._filterattrs = ['dnaHostname', 'dnaPortNum']
+        self._childobject = DNAPluginSharedConfig
+        self._basedn = basedn
+
+    def create(self, properties=None):
+        """Create an object under base DN of our entry
+
+        :param properties: Attributes for the new entry
+        :type properties: dict
+
+        :returns: DSLdapObject of the created entry
+        """
+
+        co = self._entry_to_instance(dn=None, entry=None)
+        return co.create(properties, self._basedn)
+
+
 class Plugins(DSLdapObjects):
-    """A DSLdapObjects entity which represents MEP config entry
+    """A DSLdapObjects entity which represents plugin entry
 
     :param instance: An instance
     :type instance: lib389.DirSrv
