@@ -37,12 +37,18 @@ class Task(DSLdapObject):
         self._create_objectclasses = ['top', 'extensibleObject']
         self._protected = False
         self._exit_code = None
+        self._task_log = ""
 
     def is_complete(self):
         """Return True if task is complete, else False."""
-
-        self._exit_code = self.get_attr_val("nsTaskExitCode")
-        if not self.exists() or self._exit_code is not None:
+        self._exit_code = self.get_attr_val_utf8("nsTaskExitCode")
+        self._task_log = self.get_attr_val_utf8("nsTaskLog")
+        if not self.exists():
+            self._log.debug("complete: task has self cleaned ...")
+            # The task cleaned it self up.
+            return True
+        elif self._exit_code is not None:
+            self._log.debug("complete status: %s -> %s" % (self._exit_code, self.status()))
             return True
         return False
 
@@ -56,6 +62,15 @@ class Task(DSLdapObject):
                 return None
         return None
 
+    def get_task_log(self):
+        """Return task's exit code if task is complete, else None."""
+        if self.is_complete():
+            try:
+                return (self._task_log)
+            except TypeError:
+                return None
+        return None
+
     def wait(self, timeout=120):
         """Wait until task is complete."""
 
@@ -116,7 +131,6 @@ class FixupLinkedAttributesTask(Task):
         dn = "cn=" + self.cn + "," + DN_FIXUP_LINKED_ATTIBUTES
 
         super(FixupLinkedAttributesTask, self).__init__(instance, dn)
-        self._must_attributes.extend(['basedn'])
 
 
 class MemberOfFixupTask(Task):
@@ -591,7 +605,7 @@ class Tasks(object):
                       task
                 wait: True/[False] - If True, 'index' waits for the completion
                                      of the task before to return
-        :param vlv - this task is to reindex a VVL index
+        :param vlv - this task is to reindex a VLV index
 
         :return None
 
